<?php
/**
* SEO4ALL. Extension for the phpBB Forum Software package.
*
* @copyright (c) 2017 https://thetopfew.com
* @license http://opensource.org/licenses/gpl-2.0.php GNU General Public License v2
**/

namespace thetopfew\seo4all\controller;

use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpKernel\Exception\NotFoundHttpException;

class sitemap
{
	/** @var \thetopfew\seo4all\service\sitemap_core */
	protected $core;

	/** @var \thetopfew\seo4all\service\appender */
	protected $appender;

	/** @var \phpbb\cache\service */
	protected $cache;

	/** @var \phpbb\config\config */
	protected $config;

	/** @var \phpbb\user */
	private $user;

	/**
	* Constructor
	*/
	public function __construct(
			\thetopfew\seo4all\service\appender $appender,
			\thetopfew\seo4all\service\sitemap_core $core,
			\phpbb\cache\service $cache,
			\phpbb\config\config $config,
			\phpbb\user $user)
	{
		$this->sitemap_core = $core;
		$this->cache = $cache;
		$this->config = $config;
		$this->user = $user;
		$this->appender = $appender;
	}

	/**
	* Controller for route
	*
	* @param string		$name
	* @return \Symfony\Component\HttpFoundation\Response A Symfony Response object
	*/
	public function display_sitemap()
	{
		$driver = $this->cache->get_driver();

		if (($f_xml = $driver->get("_sitemap_file")) === false)
		{
			$f_xml = $this->sitemap_core->generate_sitemap();

			if (false == $f_xml)
			{
				throw new NotFoundHttpException($this->user->lang['SEO4ALL_MAP_NO_DATA']);
			}

			$this->config->set('sitemap_url_count', $this->appender->getUrlCount());

			if ($this->config['sitemap_cache'])
			{
				$driver->put("_sitemap_file", $f_xml, 3600*$this->config['sitemap_cache']);
			}
		}

		$response = new Response($f_xml);

		$response->headers->set('Content-Type', 'application/xml');

		return $response;
	}

	public function display_sitemap_seqno_file($seqno)
	{
		$f_xml = $this->sitemap_core->sitemap_file_get_contents($seqno);

		if (false == $f_xml)
		{
			throw new NotFoundHttpException(sprintf($this->user->lang['SEO4ALL_MAP_NO_FILE'], generate_board_url() . '/store/seo4all/' . $seqno . '.xml'));
		}

		$response = new Response($f_xml);

		$response->headers->set('Content-Type', 'application/xml');

		return $response;
	}
}
