<?php
/**
* SEO4ALL. Extension for the phpBB Forum Software package.
*
* @copyright (c) 2017 https://thetopfew.com
* @license http://opensource.org/licenses/gpl-2.0.php GNU General Public License v2
**/

namespace thetopfew\seo4all\migrations;

use phpbb\db\migration\container_aware_migration;
use thetopfew\seo4all\service\constants\permissions;
use thetopfew\seo4all\service\constants\prefixes;
use thetopfew\seo4all\service\constants\tables;
use thetopfew\seo4all\service\tags_manager;
use thetopfew\seo4all\service\db_helper;

class v_init extends \phpbb\db\migration\migration
{
	public function effectively_installed()
	{
		return isset($this->config['seo4all_version']) && version_compare($this->config['seo4all_version'], '1.0.0', '>=');
	}
	
	static public function depends_on()
	{
		return array('\phpbb\db\migration\data\v320\dev');
	}

	public function update_schema()
	{
		return array(
			'add_columns'	=> array(
				$this->table_prefix . 'topics'	=> array(
					tables::TOPICDESC	=> array('VCHAR_UNI', ''),
				),
				$this->table_prefix . 'posts'	=> array(
					'post_modified'	=> array('UINT:11', 0, 'after' => 'post_delete_user'),
				),
				FORUMS_TABLE	=> array(
					'seo_topictags_enabled'	=> array('BOOL', 0),
				),
			),
			'add_tables' => array(
				$this->table_prefix . tables::TOPICTAGS	=> array(
					'COLUMNS'		=> array(
						'id'			=> array('UINT', null, 'auto_increment'),
						'topic_id'		=> array('UINT', 0),
						'tag_id'		=> array('UINT', 1),
					),
					'PRIMARY_KEY'	=> 'id',
					'KEYS'			=> array(
						'idx_topic'		=> array('INDEX', array('topic_id')),
						'idx_tag'		=> array('INDEX', array('tag_id')),
					),
				),
				$this->table_prefix . tables::TAGS => array(
					'COLUMNS'		=> array(
						'id'			=> array('UINT', null, 'auto_increment'),
						'tag'			=> array('VCHAR:30', ''),
						'count'			=> array('UINT', 0),
						'tag_lowercase'	=> array('VCHAR:30', ''),
					),
					'PRIMARY_KEY'	=> 'id',
					'KEYS'			=> array(
						'unq_tag'		=> array('UNIQUE', array('tag')),
					),
				),
			),
		);
	}
	
	public function revert_schema()
	{
		return array(
			'drop_columns'	=> array(
				$this->table_prefix . 'posts'	=> array(
					'post_modified',
				),
				$this->table_prefix . 'topics'	=> array(
					tables::TOPICDESC,
				),
				FORUMS_TABLE	=> array(
					'seo_topictags_enabled',
				),
			),
			'drop_tables'    => array(
				$this->table_prefix . tables::TOPICTAGS,
				$this->table_prefix . tables::TAGS,
			),
		);
	}

	public function update_data()
	{
		global $user;
		// even tho info_ lang files load auto, they don't during migration. this is required for lang in admin logs on add module
		$user->add_lang_ext('thetopfew/seo4all', 'acp/info_acp_seo4all');
		
		return array(
			// Current version
			array('config.add', array('seo4all_version', '1.0.0')),

			// Add permissions
			array('permission.add', array(permissions::USE_T_DESC, false)),
			array('permission.permission_set', array('ROLE_FORUM_FULL', permissions::USE_T_DESC, 'role')),
			array('permission.permission_set', array('ROLE_FORUM_STANDARD', permissions::USE_T_DESC, 'role')),
			
			array('permission.add', array(permissions::ADMIN_EDIT_TAGS)),
			array('permission.add', array(permissions::MOD_EDIT_TAGS)),
			array('permission.add', array(permissions::USE_TAGS)),
			array('permission.permission_set', array('ROLE_ADMIN_FULL', permissions::ADMIN_EDIT_TAGS)),
			array('permission.permission_set', array('ROLE_MOD_FULL', permissions::MOD_EDIT_TAGS)),
			array('permission.permission_set', array('ROLE_MOD_STANDARD', permissions::MOD_EDIT_TAGS)),
			array('permission.permission_set', array('ROLE_USER_FULL', permissions::USE_TAGS)),
			array('permission.permission_set', array('ROLE_USER_STANDARD', permissions::USE_TAGS)),
			
			// TOPIC TAGS //
			array('config.add', array(prefixes::CONFIG.'_convert_space_to_minus', 0)),
			array('config.add', array(prefixes::CONFIG.'_whitelist_enabled', 0)),
			array('config.add', array(prefixes::CONFIG.'_whitelist', '')),
			array('config.add', array(prefixes::CONFIG.'_blacklist_enabled', 0)),
			array('config.add', array(prefixes::CONFIG.'_blacklist', '')),
			array('config.add', array(prefixes::CONFIG.'_allowed_tags_regex', $user->lang('ACP_SEO4ALL_TAGS_REGEX_DEFAULT'))),
			array('config.add', array(prefixes::CONFIG.'_allowed_tags_exp_for_users', $user->lang('ACP_SEO4ALL_TAGS_REGEX_EXP_FOR_USERS_DEFAULT'))),
			array('config.add', array(prefixes::CONFIG.'_display_tags_in_viewforum', 0)),
			array('config.add', array(prefixes::CONFIG.'_display_tagcloud_on_index', 0)),
			array('config.add', array(prefixes::CONFIG.'_max_tags_in_tagcloud', 20)),
			array('config.add', array(prefixes::CONFIG.'_display_tagcount_in_tagcloud', 0)),

			// SITEMAP //
			array('config_text.add', array('sitemap_excluded', '')),
			array('config.add', array('sitemap_cache', '24')),
			array('config.add', array('sitemap_prior_0', '0.5')),
			array('config.add', array('sitemap_prior_1', '0.6')),
			array('config.add', array('sitemap_prior_2', '0.7')),
			array('config.add', array('sitemap_prior_3', '0.8')),
			array('config.add', array('sitemap_prior_4', '0.7')),
			array('config.add', array('sitemap_prior_f', '0.9')),
			array('config.add', array('sitemap_freq_0', 'daily')),
			array('config.add', array('sitemap_freq_1', 'daily')),
			array('config.add', array('sitemap_freq_2', 'weekly')),
			array('config.add', array('sitemap_freq_3', 'weekly')),
			array('config.add', array('sitemap_freq_4', 'weekly')),
			array('config.add', array('sitemap_freq_f', 'weekly')),
			array('config.add', array('sitemap_url_limit', '50000')),
			array('config.add', array('sitemap_url_count', '')),
			array('config.add', array('sitemap_batch_size', '50000')),

			// Add all ACP modules
			array('module.add', array('acp', 'ACP_CAT_DOT_MODS', 'SEO4ALL_TITLE')),
			array('module.add', array('acp', 'SEO4ALL_TITLE', array(
					'module_basename'	=> '\thetopfew\seo4all\acp\functions_module',
					'module_auth'		=> 'ext_thetopfew/seo4all && acl_a_board',
					'modes'				=> array('sitemap', 'settings', 'whitelist', 'blacklist', 'tags'),
				))
			),
		);
	}
}

class my_migration extends container_aware_migration
{
	public function update_data()
	{
		return array(
			array('custom', array(array($this, 'calc_count_tags'))),
		);
	}

	public function revert_data()
	{
		// nothing to do, because tags & descriptions are deleted anyway.
		return array();
	}

	public function calc_count_tags()
	{
		/* @var $auth \phpbb\auth\auth */
		$auth = $this->container->get('auth');
		$db_helper = new db_helper($this->db);
		$config_text = $this->container->get('config_text');
		$tags_manager = new tags_manager($this->db, $this->config, $config_text, $auth, $db_helper, $this->table_prefix);
		$tags_manager->calc_count_tags();
	}
}
